#!/usr/bin/perl

# nanakochi123456 version nyatakasan/hiratara 1st release:mnakajim

BEGIN {
	require "fatlib.pl";
};

use utf8;
use strict;
use CGI;
use Encode qw/decode decode_utf8/;
use Encode::Guess;
use PlannedBlackoutJP::Util qw/is_galapagos/;
use Text::MicroTemplate::File;
use Nana::Cache;
use HTTP::Lite;
require "common.pl";
my $cache="./cache";
my $date=&date("Y-m-d");
my $date_ja=&date("Y年m月d日(lL)");

my $tohokudenryoku_base_url="http://www.kepco.co.jp/yamasou/juyo1_kansai.csv";
my $cachefile="powerusage_kepco1";
my $debug;
my $writesokuhoufile="$cache/kepco.twitter";
my $shortfile="$cache/kepco.short";

print <<EOM;
Content-type: text/html; charset=utf8

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta http-equiv="Content-Language" content="ja" />
<meta name="keywords" content="関西電力電力使用状況">
<title>関西電力の使用状況は終了しました。 Kepco Usage display service is ended</title>
<meta name="description" content="関西電力の使用状況は終了しました。 Kepco Usage display service is ended">
</head>
<body>
関西電力の使用状況は９月２２日をもって終了しました。 <br />
今夏の電気使用状況を閲覧するには、<a href="http://k.denkiyohou.daiba.cx/">関西電力でんき予報速報サイト</a>をご覧下さい。</a><br />
<br />
Kepco Usage display service is ended<br />
<hr />
[<a href="./">戻る - Return</a>]
</body>
</html>
EOM
exit;

#&main;

sub main {
	my $query=new CGI;
	my $mode=$query->param('m');

	if($mode eq 'qr') {
		my $qr=$query->param('qr');
		my $string=$query->param('str');
		&make_qrcode($string,$qr);
		exit;
	}

	my $mobileflg;
	my $mflg=0;
	$mflg=1 if($mode=~/m/);
	my $englishflg=0;
	$englishflg=1 if($mode=~/e/);
	$mobileflg=1 if(is_galapagos(\%ENV) || $mflg);
	my $flg=$query->param('mode');

	my $cache=new Nana::Cache (
		ext=>"csv",
		files=>100,
		dir=>"./cache",
		size=>100000,
		use=>1,
		expire=>30*60,
	);

	my $http_body=$cache->read($cachefile,0);
	if($http_body eq '' || $flg eq  'update') {
		my $status;
		($status,$http_body)=&httpcl($tohokudenryoku_base_url);
		if($status eq 0) {
			$cache->write($cachefile,$http_body);
		}
	}

	$http_body=~s/\x0D\x0A|\x0D|\x0A/\x0A/g;
	$http_body=&force_decode($http_body);

	my @pwtmp=split(/\n/,$http_body);
	my $update=$pwtmp[0];
	my $kyoukyuu=$pwtmp[2];
	my $maxpower=$pwtmp[5];
	my @pw;
	for(my $i=0; $i<24; $i++) {
		$pw[$i]=$pwtmp[$i+11];
	}

	my($update_date,$update_time,$tmp)=split(/ /,$update);
	my $update_str="$update_date $update_time";
	my($kyoukyuu_kw,$kyoukyuu_time,$kyoukyuu_date,$kyoukyuu_update_time)
		=split(/,/,$kyoukyuu);
	my($maxpower_kw,$maxpower_time,$maxpower_date,$maxpower_update_time)
		=split(/,/,$maxpower);
	my $today_persent=sprintf("%.1f",($maxpower_kw+0) / ($kyoukyuu_kw+0)*100);
	my $maxupdate="$maxpower_date $maxpower_update_time";
	my @hour_refs;
	my $last_today_power;
	my $last_today_time;
	my $last_today_persent;
	for(my $i=0; $i<24; $i++) {
		my($pw_date,$pw_time,$today_power,$forecast_power,$yesterday_power)
			=split(/,/,$pw[$i]);
		if($today_power+0 ne 0) {
			$last_today_power=$today_power;
			$last_today_time=$pw_time;
			$last_today_persent=sprintf("%.1f",($today_power+0) / ($kyoukyuu_kw+0)*100);
		}
		my $persent=sprintf("%.1f",($today_power+0) / ($kyoukyuu_kw+0)*100);
		my $diff=$today_power eq 0 ? 0 : $today_power - $yesterday_power;
		push @hour_refs,  {
			date => $pw_date,
			time => $pw_time,
			today_power => $today_power,
			yesterday_power => $yesterday_power,
			forecast_power => $forecast_power,
			persent => $persent,
			diff => $diff,
		};
	}
	if(open(W,">$shortfile")) {
		my($tmp_hour,$tmp_time)=split(/:/,$last_today_time);
		my $pw_time_hour=$tmp_hour;
		my $pw_time_time=$tmp_time;
		my $pw_time_ja=sprintf("%d時%02d分",$tmp_hour,$tmp_time);
		my $short=<<EOM;
関西電力($pw_time_ja) @{[$last_today_power]}万kW / @{[$kyoukyuu_kw]}万kW (@{[$last_today_persent]}％) 
EOM
		print W $short;
		close(W);
	}
	if(open(W,">$writesokuhoufile")) {
		my($tmp_hour,$tmp_time)=split(/:/,$last_today_time);
		my $pw_time_hour=$tmp_hour;
		my $pw_time_time=$tmp_time;
		my $pw_time_ja=sprintf("%d時%02d分",$tmp_hour,$tmp_time);

		my $twitter="@{[$date]}-$pw_time_hour-$pw_time_time" . ',';
		$twitter.=<<EOM if($pw_time_hour >= 8 && $pw_time_hour <= 21);
@{[$date_ja ]} @{[$pw_time_ja]} @{[$last_today_power]}万kW / @{[$kyoukyuu_kw]}万kW (@{[$last_today_persent]}％) #jishin_power
EOM
		if($last_today_persent >= 99) {
			$twitter.=" 計画停電の可能性があります。#keikakuteiden";
		}
		print W $twitter;
		close(W);
	}
	&gzip_compress("Content-type: text/html; charset=utf-8\nCache-Control: max-age=0\nExpires: Mon, 26, Jul 1997 05:00:00 GMT");
	my $mtf = Text::MicroTemplate::File->new;
	my $template;
	if($mflg eq 1) {
		if($englishflg eq 1) {
			$template="powerusage_en_m.html";
		} else {
			$template="powerusage_jp_m.html";
		}
	} else {
		if($englishflg eq 1) {
			$template="powerusage_en.html";
		} else {
			$template="powerusage_jp.html";
		}
	}
	my($basehref, $basehost, $basepath)=&getbasehref;
	my $qrmode= $englishflg ? "me" : "mj";
	my $html = $mtf->render_file(
		$template,
		"関西電力","KEPCO ",
		"$update_date $update_time",
		$kyoukyuu_kw,$kyoukyuu_time,
		$maxpower_kw,$maxpower_time,
		$today_persent,$maxupdate,
		\@hour_refs,
		&make_link_qrcode("$basehref?m=$qrmode")
);

	if($mobileflg) {
		$html=&z2h($html);
	}

	print <<EOM;
<?xml version="1.0" encoding="UTF-8" ?>
$html$debug
EOM
	}

# QRコードの画像のリンクを作成
sub make_link_qrcode {
	my ($string) = shift;
	if(&load_module("GD") && &load_module("GD::Barcode")) {
		my($basehref, $basehost, $basepath)=&getbasehref;
		$string=&enc($string);
		return "$basehref?m=qr\&amp;str=$string";
	}
	'';
}
